@@ -1,4 +1,4 @@
-# COPYRIGHT (C) 2020-2022 Nicotine+ Team
+# COPYRIGHT (C) 2020-2022 Nicotine+ Contributors
 #
 # GNU GENERAL PUBLIC LICENSE
 #    Version 3, 29 June 2007
@@ -106,18 +106,15 @@ def check_arguments():
 
 def check_core_dependencies():
 
-    # Require Python >= 3.5
-    import sys
-    try:
-        assert sys.version_info[:2] >= (3, 5), '.'.join(
-            map(str, sys.version_info[:3])
-        )
+    # Require minimum Python version
+    python_version = (3, 5)
 
-    except AssertionError as error:
+    import sys
+    if sys.version_info < python_version:
         return _("""You are using an unsupported version of Python (%(old_version)s).
 You should install Python %(min_version)s or newer.""") % {
-            "old_version": error,
-            "min_version": "3.5"
+            "old_version": '.'.join(map(str, sys.version_info[:3])),
+            "min_version": '.'.join(map(str, python_version))
         }
 
     # Require gdbm or semidbm, for faster loading of shelves
@@ -153,34 +150,6 @@ def rescan_shares():
     return 0
 
 
-def run_headless(core, ci_mode):
-    """ Run application in headless (no GUI) mode """
-
-    import time
-
-    from pynicotine.config import config
-    from pynicotine.logfacility import log
-
-    config.load_config()
-    log.log_levels = set(["download", "upload"] + config.sections["logging"]["debugmodes"])
-
-    connect_ready = core.start()
-
-    if not connect_ready and not ci_mode:
-        return 1
-
-    connect_success = core.connect()
-
-    if not connect_success and not ci_mode:
-        return 1
-
-    while not core.shutdown:
-        time.sleep(0.2)
-
-    config.write_configuration()
-    return 0
-
-
 def run():
     """ Run application and return its exit code """
 
@@ -195,11 +164,11 @@ def run():
         import os
         import multiprocessing
 
-        # Support SSL in frozen Windows binaries
-        if sys.platform == "win32":
-            os.environ["SSL_CERT_FILE"] = os.path.join(os.path.dirname(sys.executable), "share/ssl/cert.pem")
+        # Set up paths for frozen binaries (Windows and macOS)
+        executable_folder = os.path.dirname(sys.executable)
+        os.environ["SSL_CERT_FILE"] = os.path.join(executable_folder, "lib/cert.pem")
 
-        # Support file scanning process in frozen Windows and macOS binaries
+        # Support file scanning process in frozen binaries
         multiprocessing.freeze_support()
 
     from pynicotine.logfacility import log
@@ -230,7 +199,8 @@ def run():
 
     # Run without a GUI
     if headless:
-        return run_headless(core, ci_mode)
+        from pynicotine.cli import run_cli
+        return run_cli(core, ci_mode)
 
     # Initialize GTK-based GUI
     from pynicotine.gtkgui import run_gui
