@@ -22,6 +22,8 @@
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <http://www.gnu.org/licenses/>.
 
+import time
+
 from collections import OrderedDict
 from sys import maxsize
 
@@ -81,6 +83,7 @@ class TransferList(UserInterface):
         self.users = {}
         self.paths = {}
         self.tree_users = None
+        self.last_redraw_time = 0
 
         # Use dict instead of list for faster membership checks
         self.selected_users = OrderedDict()
@@ -116,6 +119,7 @@ class TransferList(UserInterface):
 
         self.create_model()
 
+        self.h_adjustment = self.tree_view.get_parent().get_hadjustment()
         self.column_numbers = list(range(self.transfersmodel.get_n_columns()))
         self.cols = cols = initialise_columns(
             frame, transfer_type, self.tree_view,
@@ -208,13 +212,10 @@ class TransferList(UserInterface):
             GObject.TYPE_UINT64,   # (12) speed
             GObject.TYPE_UINT,     # (13) queue position
             int,                   # (14) time elapsed
-            int,                   # (15) time left
+            GObject.TYPE_UINT64,   # (15) time left
             GObject.TYPE_PYOBJECT  # (16) transfer object
         )
 
-        if GTK_API_VERSION == 3:
-            self.transfersmodel.connect("row-changed", self.on_row_changed)
-
         if self.tree_users is not None:
             self.tree_view.set_model(self.transfersmodel)
 
@@ -296,6 +297,21 @@ class TransferList(UserInterface):
         self.user_counter.set_text(humanize(len(self.users)))
         self.file_counter.set_text(humanize(len(self.transfer_list)))
 
+    def redraw_treeview(self):
+        """ Workaround for GTK 3 issue where GtkTreeView doesn't refresh changed values
+        if horizontal scrolling is present while fixed-height mode is enabled """
+
+        if GTK_API_VERSION != 3 or self.h_adjustment.get_value() <= 0:
+            return
+
+        current_time = time.time()
+
+        if (current_time - self.last_redraw_time) < 1:
+            return
+
+        self.last_redraw_time = current_time
+        self.tree_view.queue_draw()
+
     def update_model(self, transfer=None, forceupdate=False, update_parent=True):
 
         if not forceupdate and self.frame.current_page_id != self.transfer_page.id:
@@ -312,6 +328,8 @@ class TransferList(UserInterface):
         if update_parent:
             self.update_parent_rows(transfer)
 
+        self.redraw_treeview()
+
     def update_parent_rows(self, transfer=None):
 
         if self.tree_users != "ungrouped":
@@ -431,7 +449,7 @@ class TransferList(UserInterface):
             self.transfersmodel.set_value(initer, 8, self.get_helapsed(elapsed))
             self.transfersmodel.set_value(initer, 9, self.get_hleft(left))
             self.transfersmodel.set_value(initer, 14, elapsed)
-            self.transfersmodel.set_value(initer, 15, left)
+            self.transfersmodel.set_value(initer, 15, GObject.Value(GObject.TYPE_UINT64, left))
             transfer.time_elapsed = elapsed
 
         if transfer.current_byte_offset != current_bytes:
@@ -479,7 +497,7 @@ class TransferList(UserInterface):
                 self.transfersmodel.set_value(initer, 8, helapsed)
                 self.transfersmodel.set_value(initer, 9, self.get_hleft(left))
                 self.transfersmodel.set_value(initer, 14, elapsed)
-                self.transfersmodel.set_value(initer, 15, left)
+                self.transfersmodel.set_value(initer, 15, GObject.Value(GObject.TYPE_UINT64, left))
 
             if self.transfersmodel.get_value(initer, 11) != current_byte_offset:
                 percent = self.get_percent(current_byte_offset, size)
@@ -613,7 +631,7 @@ class TransferList(UserInterface):
             GObject.Value(GObject.TYPE_UINT64, speed),
             GObject.Value(GObject.TYPE_UINT, queue_position),
             elapsed,
-            left,
+            GObject.Value(GObject.TYPE_UINT64, left),
             transfer
         )
 
@@ -816,15 +834,6 @@ class TransferList(UserInterface):
         elif action == 7:  # Browse Folder
             self.on_browse_folder()
 
-    def on_row_changed(self, *_args):
-
-        if self.frame.current_page_id != self.transfer_page.id:
-            return
-
-        # Workaround for GTK 3 issue where GtkTreeView doesn't refresh changed values
-        # if horizontal scrolling is present while fixed-height mode is enabled
-        self.tree_view.queue_draw()
-
     def on_select_user_transfers(self, *args):
 
         if not self.selected_users:
