@@ -1,4 +1,4 @@
-# COPYRIGHT (C) 2021 Nicotine+ Team
+# COPYRIGHT (C) 2021-2022 Nicotine+ Contributors
 #
 # GNU GENERAL PUBLIC LICENSE
 #    Version 3, 29 June 2007
@@ -33,13 +33,23 @@ class Interests:
 
     def server_login(self):
 
-        for thing in self.config.sections["interests"]["likes"]:
-            if thing and isinstance(thing, str):
-                self.queue.append(slskmessages.AddThingILike(thing))
+        for item in self.config.sections["interests"]["likes"]:
+            if not isinstance(item, str):
+                continue
 
-        for thing in self.config.sections["interests"]["dislikes"]:
-            if thing and isinstance(thing, str):
-                self.queue.append(slskmessages.AddThingIHate(thing))
+            item = item.strip().lower()
+
+            if item:
+                self.queue.append(slskmessages.AddThingILike(item))
+
+        for item in self.config.sections["interests"]["dislikes"]:
+            if not isinstance(item, str):
+                continue
+
+            item = item.strip().lower()
+
+            if item:
+                self.queue.append(slskmessages.AddThingIHate(item))
 
         if self.ui_callback:
             self.ui_callback.server_login()
@@ -50,7 +60,9 @@ class Interests:
 
     def add_thing_i_like(self, item):
 
-        if not item and not isinstance(item, str):
+        item = item.strip().lower()
+
+        if not item:
             return False
 
         if item in self.config.sections["interests"]["likes"]:
@@ -64,7 +76,9 @@ class Interests:
 
     def add_thing_i_hate(self, item):
 
-        if not item and not isinstance(item, str):
+        item = item.strip().lower()
+
+        if not item:
             return False
 
         if item in self.config.sections["interests"]["dislikes"]:
@@ -118,25 +132,51 @@ class Interests:
         self.queue.append(slskmessages.SimilarUsers())
 
     def global_recommendations(self, msg):
+        """ Server code: 56 """
+
         if self.ui_callback:
             self.ui_callback.global_recommendations(msg)
 
     def item_recommendations(self, msg):
+        """ Server code: 111 """
+
         if self.ui_callback:
             self.ui_callback.item_recommendations(msg)
 
     def recommendations(self, msg):
+        """ Server code: 54 """
+
         if self.ui_callback:
             self.ui_callback.recommendations(msg)
 
     def similar_users(self, msg):
+        """ Server code: 110 """
+
         if self.ui_callback:
             self.ui_callback.similar_users(msg)
 
+        for user in msg.users:
+            # Request user status, speed and number of shared files
+            self.core.watch_user(user, force_update=True)
+
+    def item_similar_users(self, msg):
+        """ Server code: 112 """
+
+        if self.ui_callback:
+            self.ui_callback.item_similar_users(msg)
+
+        for user in msg.users:
+            # Request user status, speed and number of shared files
+            self.core.watch_user(user, force_update=True)
+
     def get_user_status(self, msg):
+        """ Server code: 7 """
+
         if self.ui_callback:
             self.ui_callback.get_user_status(msg)
 
     def get_user_stats(self, msg):
+        """ Server code: 36 """
+
         if self.ui_callback:
             self.ui_callback.get_user_stats(msg)
